from datetime import datetime, timedelta
import requests


class Market:
    def __init__(self, client) -> None:
        self.client = client

    def historical_options(self, symbol, start=datetime.strftime(datetime.today() - timedelta(days=1), '%Y-%m-%d'), to=datetime.strftime(datetime.today() - timedelta(days=1), '%Y-%m-%d'), token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.get('%s/historical/options/%s/%s/%s' %
                         (self.url(), symbol, start, to), headers={'Access-Token': token})
        return r.json()

    def url(self):
        return '%s%s' % (self.client.config['base_url'], 'market')



class Error(Exception):
  pass

class WrongCredentialsError(Error):
  """Exception raised for when the login credentials are wrong.

  Attributes:
      message -- explanation of the error
  """

  def __init__(self, message = 'You must use valid credentials to login!'):
    super().__init__(message)
    self.message = message

class NotLoggedInError(Error):
  """Exception raised for login errors.

  Attributes:
      message -- explanation of the error
  """

  def __init__(self, message = 'You must first login with a valid email and password using method login(email, password).'):
    super().__init__(message)
    self.message = message

class WrongParameterError(Error):
  """Exception raised for when wrong or no parameters are passed.

  Attributes:
      message -- explanation of the error
  """
  
  def __init__(self, message = 'You must pass all necessary parameters.'):
    super().__init__(message)
    self.message = message



from oplab.errors import WrongParameterError
import requests


class Domain:
    def __init__(self, client) -> None:
        self.client = client

    def url(self):
        return '%s%s' % (self.client.config['base_url'], 'domain')

    def get_portfolio_orders(self, id, token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.get('%s/portfolios/%d/orders' %
                         (self.url(), id), headers={'Access-Token': token})
        return r.json()

    def create_order(self, order, id, token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.post('%s/portfolios/%d/orders' % (self.url(), id),
                          data=order, headers={'Access-Token': token})
        return r.json()

    def update_balancing(self, balancing, id, token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.put('%s/portfolios/%d/balancings' % (self.url(), id),
                         data=balancing, headers={'Access-Token': token})
        return r.json()

    def get_portfolios(self, token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.get('%s/portfolios/' % self.url(),
                         headers={'Access-Token': token})
        return r.json()

    def get_portfolio(self, portfolio_id, token=None):
        if (portfolio_id is None):
            raise WrongParameterError
        if (token is None):
            token = self.client.get_token()
        r = requests.get('%s/portfolios/%s' % (self.url(),
                         portfolio_id), headers={'Access-Token': token})
        return r.json()

    def get_historical_data(self, symbol, amount=None, _from=None, _to=None, resolution='1d', fill='business_days', token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.get('%s/charts/data/%s/%s?amount=%d&from=%s&to=%s&fill=%s' % (self.url(), symbol, resolution, amount if amount else '',
                         _from.strftime('%Y%m%d%H%M') if _from else '', _to.strftime('%Y%m%d%H%M') if _to else '', fill), headers={'Access-Token': token})
        return r.json()

    def get_portfolio_orders(self, id, token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.get('%s/portfolios/%d/orders' %
                         (self.url(), id), headers={'Access-Token': token})
        return r.json()

    def create_order(self, order, id, token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.post('%s/portfolios/%d/orders' %
                          (self.url(), id), data=order, headers={'Access-Token': token})
        return r.json()

    def update_balancing(self, balancing, id, token=None):
        if (token is None):
            token = self.client.get_token()
        r = requests.put('%s/portfolios/%d/balancings' %
                         (self.url(), id), data=balancing, headers={'Access-Token': token})
        return r.json()



import os
from datetime import datetime, timedelta
from oplab.v3.quant import Quant
from oplab.v3.domain import Domain
from oplab.v3.market import Market
from oplab.errors import *

import requests


class Client:
    """
    Client class that implements the REST API as in: https://apidocs.oplab.com.br/
    ...

    Attributes
    ----------
    config: object
        Object with configuration regarding this client.

    Methods
    -------
    login(email = None, password = None)
        Logs in to the API using the email and password.
    get_token()
        Returns the access-token from the auth object. Must be logged in.
    """

    def __init__(self, config=None):
        self.config = config if config is not None else {
            'base_url': '%s%s/' % (os.getenv('HOST', default='https://api.oplab.com.br/'), 'v3')
        }
        self.auth = {}
        self.is_logged = False
        self.market = Market(self)
        self.domain = Domain(self)
        self.quant = Quant(self)

    def login(self, email, password):
        r = requests.post('%sdomain/users/authenticate' %
                          self.config['base_url'], {'email': email, 'password': password})
        self.auth = r.json()
        if (self.auth['access-token']):
            self.is_logged = True
            return(self.auth)
        raise WrongCredentialsError

    def get_token(self):
        if (self.is_logged):
            return self.auth['access-token']
        raise NotLoggedInError
